@@ -1,5 +1,5 @@
 #!/bin/bash
-# $Id: 651bfbfa9a0c8ecbf2c28c7a07c75cb4eca42ea4 $
+# $Id: 055497398d56ac3f3585dc7514b40b60ad7cb8d1 $
 
 COPY_BINARIES=false
 CPIO_ARGS="--quiet --null -o -H newc --owner root:root --force-local"
@@ -37,7 +37,7 @@ copy_binaries() {
 		[[ -e "${binary}" ]] \
 			|| gen_die "Binary ${binary} could not be found"
 
-		if LC_ALL=C "${LDDTREE_COMMAND}" "${binary}" 2>&1 | fgrep -q 'not found'
+		if LC_ALL=C "${LDDTREE_COMMAND}" "${binary}" 2>&1 | grep -F -q 'not found'
 		then
 			gen_die "Binary ${binary} is linked to missing libraries and may need to be re-built"
 		fi
@@ -178,6 +178,43 @@ log_future_cpio_content() {
 	print_info 3 "=================================================================" 1 0 1
 }
 
+append_devicemanager() {
+	local PN="lvm"
+	local TDIR="${TEMP}/initramfs-dm-temp"
+	if [ -d "${TDIR}" ]
+	then
+		rm -r "${TDIR}" || gen_die "Failed to clean out existing '${TDIR}'!"
+	fi
+
+	populate_binpkg ${PN}
+
+	mkdir -p "${TDIR}" || gen_die "Failed to create '${TDIR}'!"
+
+	unpack "$(get_gkpkg_binpkg "${PN}")" "${TDIR}"
+
+	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
+
+	# Delete unneeded files
+	rm -rf \
+		sbin/lvm \
+		usr/include \
+		usr/lib/device-mapper \
+		usr/lib/pkgconfig \
+		usr/lib/lib* \
+		usr/sbin/lvm \
+		usr/share
+
+	log_future_cpio_content
+	find . -print0 | "${CPIO_COMMAND}" ${CPIO_ARGS} --append -F "${CPIO_ARCHIVE}" \
+		|| gen_die "Failed to append bcache to cpio!"
+
+	cd "${TEMP}" || die "Failed to chdir to '${TEMP}'!"
+	if isTrue "${CLEANUP}"
+	then
+		rm -rf "${TDIR}"
+	fi
+}
+
 append_devices() {
 	if isTrue "${BUSYBOX}"
 	then
@@ -217,10 +254,13 @@ append_devices() {
 		dir /dev 0755 0 0
 		nod /dev/console 660 0 0 c 5 1
 		nod /dev/null 666 0 0 c 1 3
-		nod /dev/zero 666 0 0 c 1 5
+		nod /dev/random 600 0 0 c 1 8
 		nod /dev/tty0 600 0 0 c 4 0
 		nod /dev/tty1 600 0 0 c 4 1
 		nod /dev/ttyS0 600 0 0 c 4 64
+		nod /dev/ttyS1 600 0 0 c 4 65
+		nod /dev/urandom 600 0 0 c 1 9
+		nod /dev/zero 666 0 0 c 1 5
 		EOF
 
 		print_info 3 "=================================================================" 1 0 1
@@ -249,7 +289,6 @@ append_base_layout() {
 		bin \
 		dev \
 		etc \
-		etc/mdev/helpers \
 		lib \
 		lib/console \
 		lib/dracut \
@@ -263,13 +302,13 @@ append_base_layout() {
 		usr/bin \
 		usr/lib \
 		usr/sbin \
+		var/empty \
 		var/log \
+		var/run/lock \
 	; do
 		mkdir -p "${TDIR}"/${mydir} || gen_die "Failed to create '${TDIR}/${mydir}'!"
 	done
 
-	ln -s ../run var/run || gen_die "Failed to create symlink '${TDIR}/var/run' to '${TDIR}/run'!"
-
 	chmod 1777 "${TDIR}"/tmp || gen_die "Failed to chmod of '${TDIR}/tmp' to 1777!"
 
 	# In general, we don't really need lib{32,64} anymore because we now
@@ -295,6 +334,63 @@ append_base_layout() {
 	echo "proc          /proc       proc    defaults    0 0" >> "${TDIR}"/etc/fstab \
 		|| gen_die "Failed to add proc to '${TDIR}/etc/fstab'!"
 
+	print_info 2 "$(get_indent 2)>> Adding /etc/{group,passwd,shadow} ..."
+	cat >"${TDIR}"/etc/group <<-EOF
+	root:x:0:root
+	bin:x:1:root,bin,daemon
+	daemon:x:2:root,bin,daemon
+	sys:x:3:root,bin,adm
+	adm:x:4:root,adm,daemon
+	tty:x:5:
+	disk:x:6:root,adm
+	lp:x:7:lp
+	mem:x:8:
+	kmem:x:9:
+	wheel:x:10:root
+	floppy:x:11:root
+	news:x:13:news
+	uucp:x:14:uucp
+	console:x:17:
+	audio:x:18:
+	cdrom:x:19:
+	dialout:x:20:
+	tape:x:26:root
+	video:x:27:root
+	render:x:28:
+	rpc:x:32:
+	kvm:x:78:
+	usb:x:85:
+	input:x:97:
+	utmp:x:406:
+	nogroup:x:65533:
+	nobody:x:65534:
+	EOF
+
+	chmod 0644 "${TDIR}"/etc/group \
+		|| gen_die "Failed to chmod of '${TDIR}/etc/group'!"
+
+	cat >"${TDIR}"/etc/passwd <<-EOF
+	root:x:0:0:root:/root:/usr/bin/login-remote.sh
+	nobody:x:65534:65534:nobody:/var/empty:/bin/false
+	EOF
+
+	chmod 0644 "${TDIR}"/etc/passwd \
+		|| gen_die "Failed to chmod of '${TDIR}/etc/passwd'!"
+
+	echo "root:!:0:0:99999:7:::" > "${TDIR}"/etc/shadow \
+		|| gen_die "Failed to create '/etc/shadow'!"
+
+	chmod 0640 "${TDIR}"/etc/shadow \
+		|| gen_die "Failed to chmod of '${TDIR}/etc/shadow'!"
+
+	print_info 2 "$(get_indent 2)>> Adding /etc/nsswitch.conf ..."
+	cat >"${TDIR}"/etc/nsswitch.conf <<-EOF
+	# /etc/nsswitch.conf generated by genkernel
+	passwd:    files
+	shadow:    files
+	group:     files
+	EOF
+
 	print_info 2 "$(get_indent 2)>> Adding /etc/ld.so.conf ..."
 	cat >"${TDIR}"/etc/ld.so.conf <<-EOF
 	# ld.so.conf generated by genkernel
@@ -307,9 +403,27 @@ append_base_layout() {
 	date -u '+%Y-%m-%d %H:%M:%S UTC' > "${TDIR}"/etc/build_date \
 		|| gen_die "Failed to create '${TDIR}/etc/build_date'!"
 
-	echo "Genkernel $GK_V" > "${TDIR}"/etc/build_id \
+	echo "Genkernel ${GK_V}" > "${TDIR}"/etc/build_id \
 		|| gen_die "Failed to create '${TDIR}/etc/build_id'!"
 
+	cat >"${TDIR}"/etc/initrd-release <<-EOF
+	NAME="genkernel"
+	VERSION="genkernel-${GK_V}"
+	ID=genkernel
+	VERSION_ID=${GK_V}
+	PRETTY_NAME="Gentoo/Linux genkernel-${GK_V} (Initramfs)"
+	ANSI_COLOR="0;34"
+	EOF
+
+	cp -a "${GK_SHARE}"/defaults/gksosreport.sh "${TDIR}"/usr/sbin/gksosreport \
+		|| gen_die "Failed to copy '${GK_SHARE}/defaults/gksosreport.sh' to '${TDIR}/usr/sbin/gksosreport'"
+
+	chmod 0755 "${TDIR}"/usr/sbin/gksosreport \
+		|| gen_die "Failed to chmod of '${TDIR}/usr/sbin/gksosreport'!"
+
+	ln -s /proc/self/mounts "${TDIR}"/etc/mtab \
+		|| gen_die "Failed to symlink '/etc/mtab' to '/proc/self/mounts'!"
+
 	# Allow lsinitrd from dracut to process our initramfs
 	echo "$(cat "${TDIR}/etc/build_id") ($(cat "${TDIR}/etc/build_date"))" > "${TDIR}"/lib/dracut/dracut-gk-version.info \
 		|| gen_die "Failed to create '${TDIR}/lib/dracut/dracut-gk-version.info'!"
@@ -358,8 +472,8 @@ append_base_layout() {
 	isTrue "${MICROCODE_INITRAMFS}" && build_parameters+=( --microcode-initramfs ) || build_parameters+=( --no-microcode-initramfs )
 	isTrue "${RAMDISKMODULES}" && build_parameters+=( --ramdisk-modules ) || build_parameters+=( --no-ramdisk-modules )
 	isTrue "${BUSYBOX}" && build_parameters+=( --busybox ) || build_parameters+=( --no-busybox )
+	isTrue "${BCACHE}" && build_parameters+=( --bcache ) || build_parameters+=( --no-bcache )
 	isTrue "${B2SUM}" && build_parameters+=( --b2sum ) || build_parameters+=( --no-b2sum )
-	isTrue "${DISKLABEL}" && build_parameters+=( --disklabel ) || build_parameters+=( --no-disklabel )
 	isTrue "${BTRFS}" && build_parameters+=( --btrfs ) || build_parameters+=( --no-btrfs )
 	isTrue "${ISCSI}" && build_parameters+=( --iscsi ) || build_parameters+=( --no-iscsi )
 	isTrue "${MULTIPATH}" && build_parameters+=( --multipath ) || build_parameters+=( --no-multipath )
@@ -388,18 +502,8 @@ append_base_layout() {
 	dd if=/dev/zero of="${TDIR}/var/log/wtmp" bs=1 count=0 seek=0 &>/dev/null \
 		|| die "Failed to create '${TDIR}/var/log/wtmp'!"
 
-	dd if=/dev/zero of="${TDIR}/run/utmp" bs=1 count=0 seek=0 &>/dev/null \
-		|| die "Failed to create '${TDIR}/run/utmp'!"
-
-	print_info 2 "$(get_indent 2)>> Adding mdev config ..."
-	install -m 644 -t "${TDIR}"/etc "${GK_SHARE}"/mdev/mdev.conf \
-		|| gen_die "Failed to install '${GK_SHARE}/mdev/mdev.conf'!"
-
-	install -m 755 -t "${TDIR}"/etc/mdev/helpers "${GK_SHARE}"/mdev/helpers/nvme \
-		|| gen_die "Failed to install '${GK_SHARE}/mdev/helpers/nvme'!"
-
-	install -m 755 -t "${TDIR}"/etc/mdev/helpers "${GK_SHARE}"/mdev/helpers/storage-device \
-		|| gen_die "Failed to install '${GK_SHARE}/mdev/helpers/storage-device'!"
+	dd if=/dev/zero of="${TDIR}/var/run/utmp" bs=1 count=0 seek=0 &>/dev/null \
+		|| die "Failed to create '${TDIR}/var/run/utmp'!"
 
 	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
 	log_future_cpio_content
@@ -446,7 +550,7 @@ append_busybox() {
 	done
 
 	# Set up a few default symlinks
-	local required_applets='[ ash sh mknod mount uname echo cut cat'
+	local required_applets='[ ash sh mkdir mknod mount uname echo chmod cut cat touch'
 	local required_applet=
 	for required_applet in ${required_applets}
 	do
@@ -488,6 +592,58 @@ append_e2fsprogs() {
 	unpack "$(get_gkpkg_binpkg "${PN}")" "${TDIR}"
 
 	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
+	log_future_cpio_content
+	find . -print0 | "${CPIO_COMMAND}" ${CPIO_ARGS} --append -F "${CPIO_ARCHIVE}" \
+		|| gen_die "Failed to append ${PN} to cpio!"
+
+	cd "${TEMP}" || die "Failed to chdir to '${TEMP}'!"
+	if isTrue "${CLEANUP}"
+	then
+		rm -rf "${TDIR}"
+	fi
+}
+
+append_eudev() {
+	local PN=eudev
+	local TDIR="${TEMP}/initramfs-${PN}-temp"
+	if [ -d "${TDIR}" ]
+	then
+		rm -r "${TDIR}" || gen_die "Failed to clean out existing '${TDIR}'!"
+	fi
+
+	populate_binpkg ${PN}
+	populate_binpkg hwids
+
+	mkdir "${TDIR}" || gen_die "Failed to create '${TDIR}'!"
+
+	unpack "$(get_gkpkg_binpkg "${PN}")" "${TDIR}"
+	unpack "$(get_gkpkg_binpkg hwids)" "${TDIR}"
+
+	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
+
+	if isTrue "$(can_run_programs_compiled_by_genkernel)"
+	then
+		print_info 2 "$(get_indent 2)${PN}: >> Pre-generating initramfs' /etc/udev/hwdb.bin ..."
+
+		local gen_hwdb_cmd=( "${TDIR}/usr/bin/udevadm" )
+		gen_hwdb_cmd+=( hwdb --update --root "${TDIR}" )
+		print_info 3 "COMMAND: ${gen_hwdb_cmd[*]}" 1 0 1
+		eval "${gen_hwdb_cmd[@]}" || gen_die "Failed to pre-generate initramfs' /etc/udev/hwdb.bin!"
+
+		# Now that we have a pre-generated hwdb in initramfs
+		# we can delete source files
+		rm -rf usr/lib/udev/hwdb.d/
+	fi
+
+	# Delete unneeded files
+	rm -rf usr/include \
+		usr/lib/libu* \
+		usr/lib/pkgconfig \
+		usr/share
+
+	# Disable predictable network interface names in initramfs
+	echo "" > usr/lib/udev/rules.d/80-net-name-slot.rules \
+		|| gen_die "Failed to disable predictable network interface naming rule"
 
 	log_future_cpio_content
 	find . -print0 | "${CPIO_COMMAND}" ${CPIO_ARGS} --append -F "${CPIO_ARCHIVE}" \
@@ -515,7 +671,6 @@ append_b2sum() {
 	unpack "$(get_gkpkg_binpkg "${PN}")" "${TDIR}"
 
 	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
-
 	log_future_cpio_content
 	find . -print0 | "${CPIO_COMMAND}" ${CPIO_ARGS} --append -F "${CPIO_ARCHIVE}" \
 		|| gen_die "Failed to append b2sum to cpio!"
@@ -527,9 +682,9 @@ append_b2sum() {
 	fi
 }
 
-append_blkid() {
-	local PN="util-linux"
-	local TDIR="${TEMP}/initramfs-blkid-temp"
+append_bcache() {
+	local PN="bcache-tools"
+	local TDIR="${TEMP}/initramfs-bcache-temp"
 	if [ -d "${TDIR}" ]
 	then
 		rm -r "${TDIR}" || gen_die "Failed to clean out existing '${TDIR}'!"
@@ -542,13 +697,9 @@ append_blkid() {
 	unpack "$(get_gkpkg_binpkg "${PN}")" "${TDIR}"
 
 	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
-
-	# Delete unneeded files
-	rm -rf usr/
-
 	log_future_cpio_content
 	find . -print0 | "${CPIO_COMMAND}" ${CPIO_ARGS} --append -F "${CPIO_ARCHIVE}" \
-		|| gen_die "Failed to append blkid to cpio!"
+		|| gen_die "Failed to append bcache to cpio!"
 
 	cd "${TEMP}" || die "Failed to chdir to '${TEMP}'!"
 	if isTrue "${CLEANUP}"
@@ -583,6 +734,36 @@ append_unionfs_fuse() {
 	fi
 }
 
+append_util-linux() {
+	local PN="util-linux"
+	local TDIR="${TEMP}/initramfs-util-linux-temp"
+	if [ -d "${TDIR}" ]
+	then
+		rm -r "${TDIR}" || gen_die "Failed to clean out existing '${TDIR}'!"
+	fi
+
+	populate_binpkg ${PN}
+
+	mkdir -p "${TDIR}" || gen_die "Failed to create '${TDIR}'!"
+
+	unpack "$(get_gkpkg_binpkg "${PN}")" "${TDIR}"
+
+	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
+
+	# Delete unneeded files
+	rm -rf usr/
+
+	log_future_cpio_content
+	find . -print0 | "${CPIO_COMMAND}" ${CPIO_ARGS} --append -F "${CPIO_ARCHIVE}" \
+		|| gen_die "Failed to append ${PN} to cpio!"
+
+	cd "${TEMP}" || die "Failed to chdir to '${TEMP}'!"
+	if isTrue "${CLEANUP}"
+	then
+		rm -rf "${TDIR}"
+	fi
+}
+
 append_multipath() {
 	local PN=multipath-tools
 	local TDIR="${TEMP}/initramfs-${PN}-temp"
@@ -593,15 +774,39 @@ append_multipath() {
 
 	mkdir -p "${TDIR}"/etc || gen_die "Failed to create '${TDIR}/etc'!"
 
+	mkdir -p "${TDIR}"/usr/lib/udev/rules.d || gen_die "Failed to create '${TDIR}/usr/lib/udev/rules.d'!"
+
 	local libdir=$(get_chost_libdir)
+	if [[ "${libdir}" =~ ^/usr ]]
+	then
+		libdir=${libdir/\/usr/}
+	fi
 
 	copy_binaries \
-		"${TDIR}"\
+		"${TDIR}" \
 		/sbin/multipath \
 		/sbin/kpartx \
 		/sbin/mpathpersist \
-		${libdir}/multipath/lib*.so \
-		/lib/udev/scsi_id
+		${libdir}/multipath/lib*.so
+
+	local udevdir=$(get_udevdir)
+	local udevdir_initramfs="/usr/lib/udev"
+	local udev_files=( $(qlist -e sys-fs/multipath-tools:0 \
+		| grep -E -- "^${udevdir}")
+	)
+
+	if [ ${#udev_files[@]} -eq 0 ]
+	then
+		gen_die "Something went wrong: Did not found any udev-related files for sys-fs/multipath-tools!"
+	fi
+
+	local udev_files
+	for udev_file in "${udev_files[@]}"
+	do
+		local dest_file="${TDIR%/}${udev_file/${udevdir}/${udevdir_initramfs}}"
+		cp -aL "${udev_file}" "${dest_file}" \
+			|| gen_die "Failed to copy '${udev_file}' to '${dest_file}'"
+	done
 
 	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
 
@@ -650,8 +855,6 @@ append_dmraid() {
 		usr/share \
 		usr/include
 
-	mkdir -p "${TDIR}"/var/lock/dmraid || gen_die "Failed to create '${TDIR}/var/lock/dmraid'!"
-
 	log_future_cpio_content
 	find . -print0 | "${CPIO_COMMAND}" ${CPIO_ARGS} --append -F "${CPIO_ARCHIVE}" \
 		|| gen_die "Failed to append dmraid to cpio!"
@@ -717,7 +920,10 @@ append_lvm() {
 
 	# Delete unneeded files
 	rm -rf \
-		usr/lib \
+		usr/lib/device-mapper \
+		usr/lib/pkgconfig \
+		usr/lib/lib* \
+		usr/sbin/dm* \
 		usr/share \
 		usr/include
 
@@ -745,7 +951,8 @@ append_lvm() {
 		# Some LVM config options need changing, because the functionality is
 		# not compiled in:
 		sed -r -i \
-			-e '/^[[:space:]]*obtain_device_list_from_udev/s,=.*,= 0,g' \
+			-e '/^[[:space:]]*obtain_device_list_from_udev/s,=.*,= 1,g' \
+			-e '/^[[:space:]]*udev_sync/s,=.*,= 1,g' \
 			-e '/^[[:space:]]*use_lvmetad/s,=.*,= 0,g' \
 			-e '/^[[:space:]]*use_lvmlockd/s,=.*,= 0,g' \
 			-e '/^[[:space:]]*use_lvmpolld/s,=.*,= 0,g' \
@@ -835,7 +1042,6 @@ append_xfsprogs() {
 	unpack "$(get_gkpkg_binpkg "${PN}")" "${TDIR}"
 
 	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
-
 	log_future_cpio_content
 	find . -print0 | "${CPIO_COMMAND}" ${CPIO_ARGS} --append -F "${CPIO_ARCHIVE}" \
 		|| gen_die "Failed to append ${PN} to cpio!"
@@ -856,7 +1062,6 @@ append_zfs() {
 	fi
 
 	mkdir "${TDIR}" || gen_die "Failed to create '${TDIR}'!"
-	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
 
 	mkdir -p "${TDIR}"/etc/zfs || gen_die "Failed to create '${TDIR}/etc/zfs'!"
 
@@ -886,6 +1091,27 @@ append_zfs() {
 
 	copy_binaries "${TDIR}" /sbin/{mount.zfs,zdb,zfs,zpool}
 
+	local udevdir=$(get_udevdir)
+	local udevdir_initramfs="/usr/lib/udev"
+	local udev_files=( $(qlist -e sys-fs/zfs:0 \
+		| grep -E -- "^${udevdir}")
+	)
+
+	if [ ${#udev_files[@]} -eq 0 ]
+	then
+		gen_die "Something went wrong: Did not found any udev-related files for sys-fs/zfs!"
+	fi
+
+	mkdir -p "${TDIR}"/usr/lib/udev/rules.d || gen_die "Failed to create '${TDIR}/usr/lib/udev/rules.d'!"
+
+	local udev_files
+	for udev_file in "${udev_files[@]}"
+	do
+		local dest_file="${TDIR%/}${udev_file/${udevdir}/${udevdir_initramfs}}"
+		cp -aL "${udev_file}" "${dest_file}" \
+			|| gen_die "Failed to copy '${udev_file}' to '${dest_file}'"
+	done
+
 	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
 	log_future_cpio_content
 	find . -print0 | "${CPIO_COMMAND}" ${CPIO_ARGS} --append -F "${CPIO_ARCHIVE}" \
@@ -913,7 +1139,6 @@ append_btrfs() {
 	unpack "$(get_gkpkg_binpkg "${PN}")" "${TDIR}"
 
 	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
-
 	log_future_cpio_content
 	find . -print0 | "${CPIO_COMMAND}" ${CPIO_ARGS} --append -F "${CPIO_ARCHIVE}" \
 		|| gen_die "Failed to append ${PN} to cpio!"
@@ -1086,7 +1311,6 @@ append_strace() {
 
 append_overlay() {
 	cd "${INITRAMFS_OVERLAY}"  || gen_die "Failed to chdir to '${INITRAMFS_OVERLAY}'!"
-
 	log_future_cpio_content
 	find . -print0 | "${CPIO_COMMAND}" ${CPIO_ARGS} --append -F "${CPIO_ARCHIVE}" \
 		|| gen_die "Failed to append overlay to cpio!"
@@ -1382,18 +1606,10 @@ append_dropbear() {
 	cp -aL /etc/localtime "${TDIR}"/etc/ \
 		|| gen_die "Failed to copy '/etc/localtime'. Please set system's timezone!"
 
-	echo "root:x:0:0:root:/root:/usr/bin/login-remote.sh" > "${TDIR}"/etc/passwd \
-		|| gen_die "Failed to create '/etc/passwd'!"
 
 	echo "/usr/bin/login-remote.sh" > "${TDIR}"/etc/shells \
 		|| gen_die "Failed to create '/etc/shells'!"
 
-	echo "root:!:0:0:99999:7:::" > "${TDIR}"/etc/shadow \
-		|| gen_die "Failed to create '/etc/shadow'!"
-
-	echo "root:x:0:root" > "${TDIR}"/etc/group \
-		|| gen_die "Failed to create '/etc/group'!"
-
 	chmod 0755 "${TDIR}"/usr/bin/login-remote.sh \
 		|| gen_die "Failed to chmod of '${TDIR}/usr/bin/login-remote.sh'!"
 
@@ -1406,18 +1622,8 @@ append_dropbear() {
 	chmod 0755 "${TDIR}"/usr/sbin/unlock-zfs \
 		|| gen_die "Failed to chmod of '${TDIR}/usr/sbin/unlock-zfs'!"
 
-	chmod 0640 "${TDIR}"/etc/shadow \
-		|| gen_die "Failed to chmod of '${TDIR}/etc/shadow'!"
-
-	chmod 0644 "${TDIR}"/etc/passwd \
-		|| gen_die "Failed to chmod of '${TDIR}/etc/passwd'!"
-
-	chmod 0644 "${TDIR}"/etc/group \
-		|| gen_die "Failed to chmod of '${TDIR}/etc/group'!"
-
 	cd "${TDIR}" || gen_die "Failed to chdir to '${TDIR}'!"
 	log_future_cpio_content
-
 	find . -print0 | "${CPIO_COMMAND}" ${CPIO_ARGS} --append -F "${CPIO_ARCHIVE}" \
 		|| gen_die "Failed to append ${PN} to cpio!"
 
@@ -1524,11 +1730,9 @@ append_modules() {
 	local modules_dstdir="${TDIR}/lib/modules/${KV}"
 	local modules_srcdir="/lib/modules/${KV}"
 
-	print_info 2 "$(get_indent 2)modules: >> Copying modules to initramfs ..."
-
-	if [ -n "${INSTALL_MOD_PATH}" ]
+	if [ -n "${KERNEL_MODULES_PREFIX}" ]
 	then
-		modules_srcdir="${INSTALL_MOD_PATH%/}${modules_srcdir}"
+		modules_srcdir="${KERNEL_MODULES_PREFIX%/}${modules_srcdir}"
 	fi
 
 	if [ ! -d "${modules_srcdir}" ]
@@ -1541,6 +1745,8 @@ append_modules() {
 
 	cd "${modules_srcdir}" || gen_die "Failed to chdir to '${modules_srcdir}'!"
 
+	print_info 2 "$(get_indent 2)modules: >> Copying modules from '${modules_srcdir}' to initramfs ..."
+
 	local i= mymod=
 	local MOD_EXT="$(modules_kext)"
 	local n_copied_modules=0
@@ -1569,6 +1775,11 @@ append_modules() {
 	cp -ax --parents --target-directory "${modules_dstdir}" modules* 2>/dev/null \
 		|| gen_die "Failed to copy '${modules_srcdir}/modules*' to '${modules_dstdir}'!"
 
+	print_info 2 "$(get_indent 2)modules: Updating modules.dep ..."
+	local a depmod_cmd=( depmod -a -b "${TDIR}" ${KV} )
+	print_info 3 "COMMAND: ${depmod_cmd[*]}" 1 0 1
+	eval "${depmod_cmd[@]}" || gen_die "Failed to run '${depmod_cmd[*]}'!"
+
 	local group_modules= group=
 	for group_modules in ${!MODULES_*}
 	do
@@ -1641,18 +1852,14 @@ append_auxilary() {
 	done
 
 	local mylinuxrc=
-	if [ -f "${CMD_LINUXRC}" ]
+	if [ -n "${LINUXRC}" ]
 	then
-		mylinuxrc="${CMD_LINUXRC}"
+		mylinuxrc="${LINUXRC}"
 		print_info 2 "$(get_indent 2)>> Copying user specified linuxrc '${mylinuxrc}' to '/init' ..."
-		cp -aL "${mylinuxrc}" "${TDIR}"/init 2>/dev/null \
-			|| gen_die "Failed to copy '${mylinuxrc}' to '${TDIR}/init'!"
 	elif isTrue "${NETBOOT}"
 	then
 		mylinuxrc="${GK_SHARE}/netboot/linuxrc.x"
 		print_info 2 "$(get_indent 2)>> Copying netboot specific linuxrc '${mylinuxrc}' to '/init' ..."
-		cp -aL "${mylinuxrc}" "${TDIR}"/init 2>/dev/null \
-			|| gen_die "Failed to copy '${mylinuxrc}' to '${TDIR}/init'!"
 	else
 		if [ -f "${GK_SHARE}/arch/${ARCH}/linuxrc" ]
 		then
@@ -1662,10 +1869,11 @@ append_auxilary() {
 		fi
 
 		print_info 2 "$(get_indent 2)>> Copying '${mylinuxrc}' to '/init' ..."
-		cp -aL "${mylinuxrc}" "${TDIR}"/init 2>/dev/null \
-			|| gen_die "Failed to copy '${mylinuxrc}' to '${TDIR}/init'!"
 	fi
 
+	cp -aL "${mylinuxrc}" "${TDIR}"/init 2>/dev/null \
+		|| gen_die "Failed to copy '${mylinuxrc}' to '${TDIR}/init'!"
+
 	# Make sure it's executable
 	chmod 0755 "${TDIR}"/init || gen_die "Failed to chmod of '${TDIR}/init' to 0755!"
 
@@ -1770,16 +1978,17 @@ append_data() {
 }
 
 create_initramfs() {
-	local compress_ext=""
 	print_info 1 "initramfs: >> Initializing ..."
 
 	# Create empty cpio
 	CPIO_ARCHIVE="${TMPDIR}/${GK_FILENAME_TEMP_INITRAMFS}"
 	append_data 'devices' # WARNING, must be first!
 	append_data 'base_layout'
+	append_data 'util-linux'
+	append_data 'eudev'
+	append_data 'devicemanager'
 	append_data 'auxilary' "${BUSYBOX}"
 	append_data 'busybox' "${BUSYBOX}"
-	append_data 'blkid' "${DISKLABEL}"
 	append_data 'b2sum' "${B2SUM}"
 	append_data 'btrfs' "${BTRFS}"
 	append_data 'dmraid' "${DMRAID}"
@@ -1789,6 +1998,7 @@ create_initramfs() {
 	append_data 'iscsi' "${ISCSI}"
 	append_data 'luks' "${LUKS}"
 	append_data 'lvm' "${LVM}"
+	append_data 'bcache' "${BCACHE}"
 	append_data 'mdadm' "${MDADM}"
 	append_data 'modprobed'
 	append_data 'multipath' "${MULTIPATH}"
@@ -1866,21 +2076,21 @@ create_initramfs() {
 
 	cd "${TEMP}" || gen_die "Failed to chdir to '${TEMP}'"
 
-	# NOTE: We do not work with ${KERNEL_CONFIG} here, since things like
-	#       "make oldconfig" or --no-clean could be in effect.
-	if [ -s "${KERNEL_OUTPUTDIR}/.config" ]
+	local kconfig_file_used="${KERNEL_CONFIG}"
+	if isTrue "${BUILD_KERNEL}"
 	then
-		local ACTUAL_KERNEL_CONFIG="${KERNEL_OUTPUTDIR}/.config"
-	else
-		local ACTUAL_KERNEL_CONFIG="${KERNEL_CONFIG}"
+		kconfig_file_used="${KERNEL_OUTPUTDIR}/.config"
 	fi
 
-	if isTrue "$(is_gzipped "${ACTUAL_KERNEL_CONFIG}")"
+	if isTrue "$(is_gzipped "${kconfig_file_used}")"
 	then
-		# Support --kernel-config=/proc/config.gz, mainly
-		local CONFGREP=zgrep
-	else
-		local CONFGREP=grep
+		print_info 5 "Compressed kernel config '${kconfig_file_used}' found; Must decompress to temporary file ..."
+
+		local kconfig_file_tmp="${TEMP}/current_kernel.config"
+		zcat "${kconfig_file_used}" > "${kconfig_file_tmp}" \
+			|| gen_die "Failed to decompress '${kconfig_file_used}' to '${kconfig_file_tmp}'!"
+
+		kconfig_file_used="${kconfig_file_tmp}"
 	fi
 
 	if isTrue "${INTEGRATED_INITRAMFS}"
@@ -1888,18 +2098,18 @@ create_initramfs() {
 		# Explicitly do not compress if we are integrating into the kernel.
 		# The kernel will do a better job of it than us.
 		mv "${CPIO_ARCHIVE}" "${CPIO_ARCHIVE}.cpio"
-		sed -i '/^.*CONFIG_INITRAMFS_SOURCE=.*$/d' "${KERNEL_OUTPUTDIR}/.config" \
-			|| gen_die "failed to delete CONFIG_INITRAMFS_SOURCE from '${KERNEL_OUTPUTDIR}/.config'"
 
 		print_info 1 "$(get_indent 1)>> --integrated-initramfs is set; Setting CONFIG_INITRAMFS_* options ..."
 
 		[ -f "${KCONFIG_MODIFIED_MARKER}" ] && rm "${KCONFIG_MODIFIED_MARKER}"
-		kconfig_set_opt "${KERNEL_OUTPUTDIR}/.config" "CONFIG_BLK_DEV_INITRD" "y"
-		kconfig_set_opt "${KERNEL_OUTPUTDIR}/.config" "CONFIG_INITRAMFS_SOURCE" "\"${CPIO_ARCHIVE}.cpio\""
-		kconfig_set_opt "${KERNEL_OUTPUTDIR}/.config" "CONFIG_INITRAMFS_ROOT_UID" "0"
-		kconfig_set_opt "${KERNEL_OUTPUTDIR}/.config" "CONFIG_INITRAMFS_ROOT_GID" "0"
+		[ -f "${KCONFIG_REQUIRED_OPTIONS}" ] && rm "${KCONFIG_REQUIRED_OPTIONS}"
 
-		set_initramfs_compression_method "${KERNEL_OUTPUTDIR}/.config"
+		kconfig_set_opt "${kconfig_file_used}" "CONFIG_BLK_DEV_INITRD" "y"
+		kconfig_set_opt "${kconfig_file_used}" "CONFIG_INITRAMFS_SOURCE" "\"${CPIO_ARCHIVE}.cpio\""
+		kconfig_set_opt "${kconfig_file_used}" "CONFIG_INITRAMFS_ROOT_UID" "0"
+		kconfig_set_opt "${kconfig_file_used}" "CONFIG_INITRAMFS_ROOT_GID" "0"
+
+		set_initramfs_compression_method "${kconfig_file_used}"
 
 		if [ -f "${KCONFIG_MODIFIED_MARKER}" ]
 		then
@@ -1911,66 +2121,18 @@ create_initramfs() {
 	else
 		if isTrue "${COMPRESS_INITRD}"
 		then
-			local cmd_xz=$(type -p xz)
-			local cmd_lzma=$(type -p lzma)
-			local cmd_bzip2=$(type -p bzip2)
-			local cmd_gzip=$(type -p gzip)
-			local cmd_lzop=$(type -p lzop)
-			local cmd_lz4=$(type -p lz4)
-			local pkg_xz='app-arch/xz-utils'
-			local pkg_lzma='app-arch/xz-utils'
-			local pkg_bzip2='app-arch/bzip2'
-			local pkg_gzip='app-arch/gzip'
-			local pkg_lzop='app-arch/lzop'
-			local pkg_lz4='app-arch/lz4'
-			local compression
-			case ${COMPRESS_INITRD_TYPE} in
-				xz|lzma|bzip2|gzip|lzop|lz4) compression=${COMPRESS_INITRD_TYPE} ;;
-				lzo) compression=lzop ;;
-				best|fastest)
-					local tuple
-					for tuple in \
-							'CONFIG_RD_XZ    cmd_xz    xz' \
-							'CONFIG_RD_LZMA  cmd_lzma  lzma' \
-							'CONFIG_RD_BZIP2 cmd_bzip2 bzip2' \
-							'CONFIG_RD_GZIP  cmd_gzip  gzip' \
-							'CONFIG_RD_LZO   cmd_lzop  lzop' \
-							'CONFIG_RD_LZ4   cmd_lz4   lz4' \
-							; do
-						set -- ${tuple}
-						local kernel_option=${1}
-						local cmd_variable_name=${2}
-						if ${CONFGREP} -q "^${kernel_option}=y" "${ACTUAL_KERNEL_CONFIG}" && test -n "${!cmd_variable_name}" ; then
-							compression=${3}
-							[[ ${COMPRESS_INITRD_TYPE} == best ]] && break
-						fi
-					done
-					[[ -z "${compression}" ]] && gen_die "None of the initramfs compression methods we tried are supported by your kernel (config file \"${ACTUAL_KERNEL_CONFIG}\"), strange!?"
-					;;
-				*)
-					gen_die "Compression '${COMPRESS_INITRD_TYPE}' unknown"
-					;;
-			esac
+			if ! isTrue "${BUILD_KERNEL}" || isTrue "${KERNCACHE_IS_VALID}"
+			then
+				# We need to initialize COMPRESS_INITRD_TYPE in case it was set
+				# to best/fastest and validate if used kernel config can decompress
+				# set COMPRESS_INITRD_TYPE at all.
+				set_initramfs_compression_method "${kconfig_file_used}"
+			fi
 
-			# Check for actual availability
-			local cmd_variable_name=cmd_${compression}
-			local pkg_variable_name=pkg_${compression}
-			[[ -z "${!cmd_variable_name}" ]] && gen_die "Compression '${compression}' is not available. Please install package '${!pkg_variable_name}'."
-
-			local compress_ext compress_cmd
-			case ${compression} in
-				xz) compress_ext='.xz' compress_cmd="${cmd_xz} -e --check=none -z -f -9" ;;
-				lzma) compress_ext='.lzma' compress_cmd="${cmd_lzma} -z -f -9" ;;
-				bzip2) compress_ext='.bz2' compress_cmd="${cmd_bzip2} -z -f -9" ;;
-				gzip) compress_ext='.gz' compress_cmd="${cmd_gzip} -f -9" ;;
-				lzop) compress_ext='.lzo' compress_cmd="${cmd_lzop} -f -9" ;;
-				lz4) compress_ext='.lz4' compress_cmd="${cmd_lz4} -f -9 -l -q" ;;
-			esac
-
-			print_info 1 "$(get_indent 1)>> Compressing cpio data (${compress_ext}) ..."
-			print_info 3 "COMMAND: ${compress_cmd} ${CPIO_ARCHIVE}" 1 0 1
-			${compress_cmd} "${CPIO_ARCHIVE}" || gen_die "Compression (${compress_cmd}) failed"
-			mv -f "${CPIO_ARCHIVE}${compress_ext}" "${CPIO_ARCHIVE}" || gen_die "Rename failed"
+			print_info 1 "$(get_indent 1)>> Compressing cpio data (${GKICM_LOOKUP_TABLE_EXT[${COMPRESS_INITRD_TYPE}]}) ..."
+			print_info 3 "COMMAND: ${GKICM_LOOKUP_TABLE_CMD[${COMPRESS_INITRD_TYPE}]} ${CPIO_ARCHIVE}" 1 0 1
+			${GKICM_LOOKUP_TABLE_CMD[${COMPRESS_INITRD_TYPE}]} "${CPIO_ARCHIVE}" || gen_die "Initramfs compression using '${GKICM_LOOKUP_TABLE_CMD[${COMPRESS_INITRD_TYPE}]}' failed"
+			mv -f "${CPIO_ARCHIVE}${GKICM_LOOKUP_TABLE_EXT[${COMPRESS_INITRD_TYPE}]}" "${CPIO_ARCHIVE}" || gen_die "Rename failed"
 		else
 			print_info 3 "$(get_indent 1)>> --no-compress-initramfs is set; Skipping compression of initramfs ..."
 		fi
@@ -1981,9 +2143,9 @@ create_initramfs() {
 		## be before the other cpio archives in the stream.
 		if isTrue "${MICROCODE_INITRAMFS}"
 		then
-			local cfg_CONFIG_MICROCODE=$(kconfig_get_opt "${KERNEL_OUTPUTDIR}"/.config CONFIG_MICROCODE)
-			local cfg_CONFIG_MICROCODE_INTEL=$(kconfig_get_opt "${KERNEL_OUTPUTDIR}"/.config CONFIG_MICROCODE_INTEL)
-			local cfg_CONFIG_MICROCODE_AMD=$(kconfig_get_opt "${KERNEL_OUTPUTDIR}"/.config CONFIG_MICROCODE_AMD)
+			local cfg_CONFIG_MICROCODE=$(kconfig_get_opt "${kconfig_file_used}" CONFIG_MICROCODE)
+			local cfg_CONFIG_MICROCODE_INTEL=$(kconfig_get_opt "${kconfig_file_used}" CONFIG_MICROCODE_INTEL)
+			local cfg_CONFIG_MICROCODE_AMD=$(kconfig_get_opt "${kconfig_file_used}" CONFIG_MICROCODE_AMD)
 			print_info 1 "$(get_indent 1)>> Adding early-microcode support ..."
 			local UCODEDIR="${TEMP}/ucode_tmp/kernel/x86/microcode/"
 			mkdir -p "${UCODEDIR}" || gen_die "Failed to create '${UCODEDIR}'!"
